------------------------------
--      Are you local?      --
------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Postal")

---------------------------------
--      Addon Declaration      --
---------------------------------

Postal = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1")

Postal.revision = tonumber(string.sub("$Revision: 56101 $", 12, -3))

Postal:SetModuleMixins("AceEvent-2.0", "AceHook-2.1")

Postal.defaults = {}

Postal.consoleOptions = {
	type = "group",
	handler = Postal,
	args = {},
}

Postal:RegisterDB("PostalDB", "PostalDBPerChar")
Postal:RegisterDefaults("profile", Postal.defaults)
Postal:RegisterChatCommand({ "/Postal" }, Postal.consoleOptions, "POSTAL")

------------------------------
--      Initialization      --
------------------------------

function Postal:OnInitialize()
	if not self.version then self.version = GetAddOnMetadata("Postal", "Version") end
	local rev, name = self.revision, nil
	for name, module in self:IterateModules() do
		rev = math.max(rev, module.revision)
	end
	self.version = (self.version or "1.0") .. " |cffff8888r" .. rev .. "|r"
end

function Postal:OnDisable()
	local name, module = nil, nil
	for name, module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end
end

--------------------------------
--      Module Prototype      --
--------------------------------

Postal.modulePrototype.core = Postal
Postal.modulePrototype.revision = 1

function Postal.modulePrototype:OnInitialize()
	-- Unconditionally register, this shouldn't happen from any other place
	-- anyway.
	self.core:RegisterModule(self.name, self)
	hooksecurefunc(self, "OnEnable", self.OnEnable_Common)
end

function Postal.modulePrototype:OnEnable_Common()
	if self.MAIL_SHOW and MailFrame:IsVisible() then
		self:MAIL_SHOW()
	end
end

function Postal.modulePrototype:IsRegistered()
	return self.registered
end

-------------------------------
--      Module Handling      --
-------------------------------

function Postal:RegisterModule(name, module)
	if module:IsRegistered() then
		error(string.format("%q is already registered.", name))
		return
	end

	if module.db and module.RegisterDefaults and type(module.RegisterDefaults) == "function" then
		module:RegisterDefaults("char", module.defaults or {})
	else
		self:RegisterDefaults(name, "char", module.defaults or {})
	end

	if not module.db then module.db = self:AcquireDBNamespace(name) end

	if module.consoleOptions then
		local m = module.consoleCmd or name
		-- if the consoleoption already exists we merge in the data otherwise we create a new option
		if self.consoleOptions.args[m] then
			for k,v in pairs(module.consoleOptions.args) do self.consoleOptions.args[m].args[k] = v end
		else
			self.consoleOptions.args[m] = module.consoleOptions
		end
	end

	module.registered = true
	if module.OnRegister and type(module.OnRegister) == "function" then
		module:OnRegister()
	end
end

-------------------------------
--      Utility Functions    --
-------------------------------

-- Disable Inbox Clicks
function Postal:DisableInbox(disable)
	if disable then
		if not self:IsHooked("InboxFrame_OnClick") then
			self:Hook("InboxFrame_OnClick", function() end, true)
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(1)
			end
		end
	else
		if self:IsHooked("InboxFrame_OnClick") then
			self:Unhook("InboxFrame_OnClick")
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(nil)
			end
		end
	end
end

function Postal:DisableAttachments(disable)
	if disable then
		OpenMailMoneyButtonIconTexture:SetDesaturated(1)
		OpenMailPackageButtonIconTexture:SetDesaturated(1)
		if not self:IsHooked(OpenMailMoneyButton, "OnClick") then
			self:HookScript(OpenMailMoneyButton, "OnClick", function() end)
		end
		if not self:IsHooked(OpenMailPackageButton, "OnClick") then
			self:HookScript(OpenMailPackageButton, "OnClick", function() end)
		end
	else
		if (OpenMailMoneyButtonIconTexture ~= nil) then
			OpenMailMoneyButtonIconTexture:SetDesaturated(nil)
		end
		
		if (OpenMailPackageButtonIconTexture ~= nil) then
			OpenMailPackageButtonIconTexture:SetDesaturated(nil)
		end
		
		if self:IsHooked("OpenMailMoneyButton_OnClick") then
			self:Unhook("OpenMailMoneyButton_OnClick")
		end
		if self:IsHooked("OpenMailPackageButton_OnClick") then
			self:Unhook("OpenMailPackageButton_OnClick")
		end
	end
end

-- Remove a table entry by the value as key, from wowwiki
function Postal:tremovebyval(tab, val)
        for k,v in pairs(tab) do
                if v == val then
                        table.remove(tab, k)
                        return true
                end
        end
        return false
end

--------------------------------
-- Common Inventory Functions --
--------------------------------
function Postal:SpaceCheck()
	for bag = 0, 4 do
		local size = GetContainerNumSlots(bag)
		if size ~= nil and size > 0 then
			for slot = 1, size do
				if not GetContainerItemInfo(bag, slot) then return true end
			end
		end
	end
end

function Postal:GetSlotTable()
	local invindex = {}
	for bag = 0, 4 do
		local size = GetContainerNumSlots(bag)
		if size ~= nil and size > 0 then
			invindex[bag] = {}
			for slot = 1, size do
				if GetContainerItemInfo(bag, slot) then invindex[bag][slot] = true end
			end
		end
	end
	return invindex
end

function Postal:FindTakenSlot(invindex)
	for bag = 0, 4 do
		local size = GetContainerNumSlots(bag)
		if type(size) == "number" and size > 0 then
			for slot = 1, size do
				if GetContainerItemInfo(bag, slot) and not invindex[bag][slot] then
					return bag, slot
				end
			end
		end
	end
end

---------------------------
-- Common Mail Functions --
---------------------------

function Postal:PurgeMail(index, unusedflag)
	self.postalindex = index
	self.hasmoney = 0
	self.hasitem = nil
	self.autodelete = nil

	--self.disableinbox = nil
	local msgSubject, msgMoney, msgCOD, _, msgItem, _, _, msgText = select(4, GetInboxHeaderInfo(index))

	if msgCOD and msgCOD > 0 then self:TriggerEvent("Postal10_MailSkipped", index) return end

	if msgSubject:find(AUCTION_SOLD_MAIL_SUBJECT:sub(1,10))
	or msgSubject:find(AUCTION_OUTBID_MAIL_SUBJECT:sub(1,10))
	or msgSubject:find(AUCTION_REMOVED_MAIL_SUBJECT:sub(1,10))
	or msgSubject:find(AUCTION_EXPIRED_MAIL_SUBJECT:sub(1,10))
	or msgSubject:find(AUCTION_WON_MAIL_SUBJECT:sub(1,10))
	then
		self.autodelete = 1
	end

	if not msgText and msgItem then
		self.autodelete = 1
	end

	self.hasmoney = msgMoney
	self.hasitem = msgItem

	if self.hasitem then
		self:TakeItem()
	elseif self.hasmoney > 0 then
		self:TakeMoney()
	else
		self:DeleteMail()
	end
end

function Postal:TakeItem(index)
	if not index and self.postalindex then
		index = self.postalindex
	end
	if not Postal:SpaceCheck() then
		self:TriggerEvent("Postal10_MailSkipped", index)
		return
	end
	self.skipitem = nil
	--self:DisableInbox(1)
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "GotItem")
	self:RegisterEvent("BAG_UPDATE", "GotItem")
	self:RegisterEvent("UI_ERROR_MESSAGE")
	self:RegisterEvent("MAIL_CLOSED")
	TakeInboxItem(index)
end

function Postal:GotItem()
	self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
	self:UnregisterEvent("BAG_UPDATE")
	self:UnregisterEvent("UI_ERROR_MESSAGE")
	self:UnregisterEvent("MAIL_CLOSED")
	--self.hasitem = nil
	if self.skipitem then
		self:TriggerEvent("Postal10_MailSkipped", self.postalindex)
		self.skipitem = nil
		--self:DisableInbox(nil)
	elseif self.hasitem > 1 then
		local numItemsLeft = select(8, GetInboxHeaderInfo(self.postalindex))
		if numItemsLeft == self.hasitem - 1 then
			-- The UNIT_INVENTORY_CHANGED event that triggered this is indeed
			-- from retrieving mail rather than some other inventory change,
			-- so take the next item
			self.hasitem = numItemsLeft
			self:TakeItem()
		else
			-- The UNIT_INVENTORY_CHANGED event that triggered wasn't from
			-- retrieving mail, so reregister these events for the next one
			-- that will come from retrieving the mail.
			self:RegisterEvent("UNIT_INVENTORY_CHANGED", "GotItem")
			self:RegisterEvent("BAG_UPDATE", "GotItem")
			self:RegisterEvent("UI_ERROR_MESSAGE")
			self:RegisterEvent("MAIL_CLOSED")
		end
	elseif self.hasmoney > 0 then
		self:TakeMoney(self.postalindex)
	elseif self.autodelete then
		self:ScheduleEvent("Postal10_MailPurged", 1, self.postalindex)
		--self:DisableInbox(nil)
	else
		self:DeleteMail(self.postalindex)
	end
end

function Postal:TakeMoney(index)
	if not index and self.postalindex then
		index = self.postalindex
	end
	--if not self.inboxdisabled then self:DisableInbox(1) end
	self:RegisterEvent("PLAYER_MONEY")
	TakeInboxMoney(index)
end

function Postal:GotMoney()
	if not self.hasitem and not self.autodelete then
		self:DeleteMail()
	elseif self.hasitem then
		self:TriggerEvent("Postal10_MailSkipped", self.postalindex)
		--self:DisableInbox(nil)
	else
		self:ScheduleEvent("Postal10_MailPurged", 1, self.postalindex)
		--self:DisableInbox(nil)
	end
end

function Postal:DeleteMail(index)
	if not index and self.postalindex then
		index = self.postalindex
	end
	local msgMoney, _, _, msgItem = select(5, GetInboxHeaderInfo(index))
	--if not self.inboxdisabled then self:DisableInbox(1) end
	if msgMoney == 0 and not msgItem then
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		DeleteInboxItem(index)
	else
		self:TriggerEvent("Postal10_MailSkipped", index)
	end
end

function Postal:MAIL_INBOX_UPDATE()
	--self:DisableInbox(nil)
	self:UnregisterEvent("MAIL_INBOX_UPDATE")
	self:ScheduleEvent("Postal10_MailPurged", 1, self.postalindex)
end

function Postal:ReturnMail(index)
	--self:DisableInbox(1)
	local wasReturned, _, canReply = select(10, GetInboxHeaderInfo(index))
	if not wasReturned and canReply then
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		ReturnInboxItem(index)
	else
		self:TriggerEvent("Postal10_MailSkipped", index)
	end
end

function Postal:PLAYER_MONEY()
	self:UnregisterEvent("PLAYER_MONEY")
	self:GotMoney()
end

function Postal:MAIL_CLOSED()
	self.skipitem = 1
	self:GotItem()
end

function Postal:UI_ERROR_MESSAGE(error_message)
	if error_message == ERR_INV_FULL
	or error_message == ERR_ITEM_MAX_COUNT
	or error_message == ERR_MAIL_DATABASE_ERROR then
		self.skipitem = 1
		self:GotItem()
	end
end
